/**
 *  PI Calculator CUI Frond-End
 */

#include "pi.hxx"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int usage()
{
	char app_title[sizeof(__FILE__)];
	if (char *p = strrchr(strcpy(app_title, __FILE__), '.'))
		*p = '\0';
	fprintf(stderr, "Usage: %s [Precision] [Filename]\n", app_title);
	return EXIT_SUCCESS;
}

int error(const char *msg)
{
	fprintf(stderr, "Error: %s\n", msg);
	return EXIT_FAILURE;
}

int get_precision(int argc, char *argv[])
{
	int prec = -1;
	if (argc >= 2) {
		prec = atoi(argv[1]);
		if (prec <= 0)
			error("Precision must be a positive value");
	}
	while (prec <= 0) {
		char buf[80];
		fprintf(stderr, "Precision: ");
		fgets(buf, sizeof(buf), stdin);
		if (sscanf(buf, "%d", &prec) != 1 || prec <= 0)
			error("Precision must be a positive value");
	}
	return prec;
}

int main(int argc, char *argv[])
{
	try {
		if (argc > 3)
			return usage();
		
		int prec = get_precision(argc, argv);
		
		pi_calculator pi;
		if (!pi.create(prec))
			throw "pi.create() failed";
		int n = pi.start();
		if (n == 0)
			throw "pi.start() failed";
		const int LEN = 32;
		int r, c_prev = -1, p_prev = -1;
		while ((r = pi.do_step()) != -1) {
			int c = LEN * r / n;
			int p = 100 * r / n;
			if (c != c_prev || p != p_prev) {
				fprintf(stderr, "\rCalculating ");
				for (int i = 0; i < LEN; i++)
					fputc(i < c ? '.' : ' ', stderr);
				fprintf(stderr, " [%3d%%]", p);
				c_prev = c;
				p_prev = p;
			}
		}
		fprintf(stderr, "\rCalculating ");
		for (int i = 0; i < LEN; i++)
			fputc('.', stderr);
		fprintf(stderr, " [Done]\n", stderr);
		if (argc == 3) {
			int len = pi.tostr(NULL);
			char *buf = new char[len];
			pi.tostr(buf);
			FILE *fp = fopen(argv[2], "wb");
			if (!fp)
				throw "file creation failed";
			fwrite(buf, 1, strlen(buf), fp);
			fclose(fp);
			delete[] buf;
		} else {
			int len = pi.tostr(NULL, false);
			char *buf = new char[len];
			pi.tostr(buf, false);
			printf("pi = ");
			puts(buf);
			delete[] buf;
		}
	} catch (const char *msg) {
		return error(msg);
	} catch (...) {
		return error("out of memory");
	}
	return EXIT_SUCCESS;
}
