/**
 *  PI Calculator
 */

#ifndef PI_HXX
#define PI_HXX

#include <string.h>

class pi_calculator
{
	enum { O = 5 };
public:
	explicit pi_calculator(int prec = 0)
		: p(NULL), t(NULL), q(-1), a(-1), n(0)
	{
		if (prec > 0)
			create(prec);
	}
	virtual ~pi_calculator()
	{
		destroy();
	}
	void destroy()
	{
		delete[] p;
		delete[] t;
		p = t = NULL;
		q = a = -1;
	}
	bool create(int prec)
	{
		if (prec < 0)
			return false;
		destroy();
		prec += O;
		p = new char[prec + 1];
		t = new char[prec + 1];
		if (!p || !t)
			return false;
		q = prec;
		return true;
	}
	bool reset()
	{
		if (q < 0)
			return false;
		memset(p, 0, q + 1);
		memset(t, 0, q + 1);
		a = -1;
		return true;
	}
	int start()
	{
		if (!reset())
			return 0;
		arctan_init(a = 2);
		return q * 2 + 1;
	}
	int do_step()
	{
		if (a < 2)
			return 0;
		int r = arctan_do(a);
		if (r < 0) {
			if (a == 2) {
				arctan_init(a = 3);
				return q;
			}
			mul4();
			a = -1;
			return -1;
		}
		return q * (a - 2) + r;
	}
	int tostr(char *buf, bool separate = true)
	{
		if (q < 0)
			return 0;
		int digit = q - O;
		if (!buf) {
			return separate
				? 4 + digit + digit / 10 + digit / 100 + 1
				: 2 + digit + 1;
		}
		*buf++ = '0' + p[0];
		*buf++ = '.';
		if (separate) {
			*buf++ = '\r';
			*buf++ = '\n';
		}
		for (int i = 1; i <= digit; i++) {
			*buf++ = '0' + p[i];
			if (separate) {
				if (i % 10 == 0) {
					if (i % 100 == 0) {
						*buf++ = '\r';
						*buf++ = '\n';
					} else {
						*buf++ = ' ';
					}
				}
			}
		}
		*buf = '\0';
		return separate
			? 4 + digit + digit / 10 + digit / 100 + 1
			: 2 + digit + 1;
	}
private:
	void add()
	{
		for (int i = q; i >= 0; i--) {
			p[i] += t[i];
			if (p[i] >= 10) {
				p[i] -= 10;
				p[i - 1]++;
			}
		}
	}
	void sub()
	{
		for (int i = q; i >= 0; i--) {
			if (p[i] < t[i]) {
				p[i] += 10;
				p[i - 1]--;
			}
			p[i] -= t[i];
		}
	}
	void mul(int m)
	{
		int c = 0;
		for (int i = q; i >= 0; i--) {
			int b = t[i] * m + c;
			int d = b % 10;
			    c = b / 10;
			t[i]  = d;
		}
	}
	void div(int d)
	{
		int r = 0;
		for (int i = 0; i <= q; i++) {
			int b = 10 * r + t[i];
			int u = b / d;
			    r = b % d;
			t[i]  = u;
		}
	}
	void mul4()
	{
		int c = 0;
		for (int i = q; i >= 0; i--) {
			int d = (p[i] * 4 + c) % 10;
			    c = (p[i] * 4 + c) / 10;
			p[i]  = d;
		}
	}
	int diff()
	{
		int i;
		for (i = 0; i <= q; i++) {
			if (t[i] != 0)
				return i;
		}
		return -1;
	}
	void arctan_init(int s)
	{
		t[0] = 1;
		div(s);
		add();
		n = 1;
	}
	int arctan_do(int s)
	{
		mul(n);
		div(s * s);
		div(n += 2);
		if ((n & 2) == 0)
			add();
		else
			sub();
		return diff();
	}
	char *p, *t;
	int q, a, n;
};

#endif//PI_HXX
